package benchmark.sensornetwork.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.io.File;

import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;

import benchmark.sensornetwork.SensorDCSPGenerator;
import gui.generators.Generator;
import static gui.Gui.*;
import util.Commands;
/**
 * SensorDCSP generator panel
 * 
 * @authors 
 * <a href="mailto:ezzahir@ensa-agadir.ac.ma?subject=DisChoco &cc=wahbi@lirmm.fr &cc=bessiere@lirmm.fr">
 * Redoaune Ezzahir</a> and 
 * <a href="mailto:wahbi@lirmm.fr?subject=DisChoco &cc=ezzahir@ensa-agadir.ac.ma &cc=bessiere@lirmm.fr">
 * Mohamed Wahbi</a>
 *
 */
public class SensorNetworkGeneratorPanel extends Generator {

	/**
	 * 
	 */
	private static final long serialVersionUID = 7148638327820325521L;
	public JLabel nbInstancesLabel = null;
	public JLabel nbSensorsLabel = null;
	public JLabel nbTargetsLabel = null;
	public JLabel Pvlabel = null;
	public JLabel Pclabel = null;
	public JLabel fromLabel = null;
	public JLabel toLabel = null;
	public JLabel stepLabel = null;
	private JSpinner minPc = null;
	private JSpinner maxPc = null;
	private JSpinner stepPc = null;
	private JSpinner minPv = null;
	private JSpinner maxPv = null;
	private JSpinner stepPv = null;
	private JSpinner nbInstances = null;
	private JSpinner nbSensors = null;
	private JSpinner nbTargets = null;
	private JPanel gridPanel = null;
	private JSpinner x_axis = null;
	private JSpinner y_axis = null;
	private JLabel x_axisLabel = null;
	private JLabel y_axisLabel = null;
	private JLabel KcLabel = null;
	private JLabel KvLabel = null;
	private JSpinner minKc = null;
	private JSpinner minKv = null;
	private JSpinner maxKc = null;
	private JSpinner stepKc = null;
	private JSpinner maxKv = null;
	private JSpinner stepKv = null;
	private JComboBox type = null;
	private JComboBox format = null;

	/**
	 * This is the default constructor
	 */
	public SensorNetworkGeneratorPanel() {
		super();
		this.add(this.getGridPanel(), BorderLayout.CENTER);
	}

	/**
	 * This method initializes gridPanel
	 * 
	 * @return simulation.gui.RandomTKCPanel
	 */
	protected JPanel getGridPanel() {
		if (gridPanel == null) {
			this.initializeLabels();
			gridPanel = new JPanel();
			gridPanel.setLayout(null);
			gridPanel.setPreferredSize(new Dimension(600, 375));
			gridPanel.setMinimumSize(new Dimension(600, 375));
			gridPanel.add(nbInstancesLabel, null);
			gridPanel.add(nbSensorsLabel, null);
			gridPanel.add(nbTargetsLabel, null);
			gridPanel.add(getNbTargets(), null);
			gridPanel.add(Pvlabel, null);
			gridPanel.add(Pclabel, null);
			gridPanel.add(fromLabel, null);
			gridPanel.add(toLabel, null);
			gridPanel.add(stepLabel, null);
			gridPanel.add(getMinPv(), null);
			gridPanel.add(getMaxPv(), null);
			gridPanel.add(getStepPv(), null);
			gridPanel.add(getMinPc(), null);
			gridPanel.add(getMaxPc(), null);
			gridPanel.add(getStepPc(), null);
			gridPanel.add(getNbInstances(), null);
			gridPanel.add(getNbSensors(), null);
			gridPanel.add(getX_axis(), null);
			gridPanel.add(getY_axis(), null);
			gridPanel.add(x_axisLabel, null);
			gridPanel.add(y_axisLabel, null);
			gridPanel.add(getType(), null);
			gridPanel.add(getFormat(), null);
			gridPanel.add(KcLabel, null);
			gridPanel.add(KvLabel, null);
			gridPanel.add(getMinKc(), null);
			gridPanel.add(getMinKv(), null);
			gridPanel.add(getMaxKc(), null); // Generated
			gridPanel.add(getStepKc(), null); // Generated
			gridPanel.add(getMaxKv(), null); // Generated
			gridPanel.add(getStepKv(), null); // Generated
		}
		return gridPanel;
	}

	private void initializeLabels() {
		nbInstancesLabel = new JLabel("Number of instances", JLabel.RIGHT);
		nbInstancesLabel.setBounds(new Rectangle(200, 40, 150, 22));
		x_axisLabel = new JLabel("Number of x-axis", JLabel.LEFT);
		x_axisLabel.setBounds(new Rectangle(65, 75, 120, 22));
		y_axisLabel = new JLabel("Number of y-axis", JLabel.LEFT);
		y_axisLabel.setBounds(new Rectangle(310, 75, 120, 22));
		nbSensorsLabel = new JLabel("Number of Sensors", JLabel.RIGHT);
		nbSensorsLabel.setBounds(new Rectangle(65, 110, 120, 22));
		nbTargetsLabel = new JLabel("Number of targets", JLabel.RIGHT);
		nbTargetsLabel.setBounds(new Rectangle(310, 110, 120, 22));
		KcLabel = new JLabel("Kc-restricted compatibility", JLabel.RIGHT);
		KcLabel.setBounds(new Rectangle(30, 185, 185, 22));
		KvLabel = new JLabel("Kv-restricted visibility", JLabel.RIGHT);
		KvLabel.setBounds(new Rectangle(30, 220, 185, 22));
		Pclabel = new JLabel("Density of communication (Pc)", JLabel.RIGHT);
		Pclabel.setBounds(new Rectangle(20, 255, 195, 22));
		Pvlabel = new JLabel("Density of visibility (Pv)", JLabel.RIGHT);
		Pvlabel.setBounds(new Rectangle(20, 290, 195, 22));
		fromLabel = new JLabel("From", JLabel.CENTER);
		fromLabel.setBounds(new Rectangle(225, 150, 50, 22));
		toLabel = new JLabel("To", JLabel.CENTER);
		toLabel.setBounds(new Rectangle(320, 150, 60, 22));
		stepLabel = new JLabel("Step", JLabel.CENTER);
		stepLabel.setBounds(new Rectangle(420, 150, 60, 22));
	}

	/**
	 * This method initializes nbTargets
	 * 
	 * @return javax.swing.JSpinner
	 */
	private JSpinner getNbTargets() {
		if (nbTargets == null) {
			nbTargets = new JSpinner(new SpinnerNumberModel(5, 3, 100, 1));
			nbTargets.setBounds(new Rectangle(440, 110, 60, 22));
		}
		return nbTargets;
	}

	/**
	 * This method initializes nbSensors
	 * 
	 * @return javax.swing.JSpinner
	 */
	private JSpinner getNbSensors() {
		if (nbSensors == null) {
			nbSensors = new JSpinner(new SpinnerNumberModel(25, 5, 1000, 1));
			nbSensors.setBounds(new Rectangle(200, 110, 60, 22));
		}
		return nbSensors;
	}

	/**
	 * This method initializes minPv
	 * 
	 * @return javax.swing.JSpinner
	 */
	public JSpinner getMinPv() {
		if (minPv == null) {
			minPv = new JSpinner(new SpinnerNumberModel(0.10, 0.01, 1.00, 0.01));
			minPv.setBounds(new Rectangle(220, 290, 60, 22));
		}
		return minPv;
	}

	/**
	 * This method initializes maxPv
	 * 
	 * @return javax.swing.JTextField
	 */
	public JSpinner getMaxPv() {
		if (maxPv == null) {
			maxPv = new JSpinner(new SpinnerNumberModel(0.90, 0.01, 1.00, 0.01));
			maxPv.setBounds(new Rectangle(320, 290, 60, 22));
		}
		return maxPv;
	}

	/**
	 * This method initializes stepPv
	 * 
	 * @return javax.swing.JTextField
	 */
	public JSpinner getStepPv() {
		if (stepPv == null) {
			stepPv = new JSpinner(new SpinnerNumberModel(0.05, 0.01, 1.00, 0.01));
			stepPv.setBounds(new Rectangle(420, 290, 60, 22));
		}
		return stepPv;
	}

	/**
	 * This method initializes minPc
	 * 
	 * @return javax.swing.JTextField
	 */
	public JSpinner getMinPc() {
		if (minPc == null) {
			minPc = new JSpinner(new SpinnerNumberModel(0.2, 0.01, 1, 0.01));
			minPc.setBounds(new Rectangle(220, 255, 60, 22));
		}
		return minPc;
	}

	/**
	 * This method initializes maxPc
	 * 
	 * @return javax.swing.JTextField
	 */
	public JSpinner getMaxPc() {
		if (maxPc == null) {
			maxPc = new JSpinner(new SpinnerNumberModel(0.8, 0.01, 1, 0.01));
			maxPc.setBounds(new Rectangle(320, 255, 60, 22));
		}
		return maxPc;
	}

	/**
	 * This method initializes stepPc
	 * 
	 * @return javax.swing.JTextField
	 */
	public JSpinner getStepPc() {
		if (stepPc == null) {
			stepPc = new JSpinner(new SpinnerNumberModel(0.05, 0.01, 1, 0.01));
			stepPc.setBounds(new Rectangle(420, 255, 60, 22));
		}
		return stepPc;
	}

	public void generate(File file) {
		int nbInstances = (Integer) (this.getNbInstances().getValue());
		int x_axis = (Integer) (this.getX_axis().getValue());
		int y_axis = (Integer) (this.getY_axis().getValue());
		int nbSensors = (Integer) (this.getNbSensors().getValue());
		int nbTargets = (Integer) (this.getNbTargets().getValue());
		int minKc = (Integer) (this.getMinKc().getValue());
		int maxKc = (Integer) (this.getMaxKc().getValue());
		int stepKc = (Integer) (this.getStepKc().getValue());
		int minKv = (Integer) (this.getMinKv().getValue());
		int maxKv = (Integer) (this.getMaxKv().getValue());
		int stepKv = (Integer) (this.getStepKv().getValue());
		int minPv = (int) ((Double) (this.minPv.getValue()) * 100);
		int maxPv = (int) ((Double) (this.maxPv.getValue()) * 100);
		int stepPv = (int) ((Double) (this.stepPv.getValue()) * 100);
		int minPc = (int) ((Double) (this.minPc.getValue()) * 100);
		int maxPc = (int) ((Double) (this.maxPc.getValue()) * 100);
		int stepPc = (int) ((Double) (this.stepPc.getValue()) * 100);
		if (stepPc > 100) {
			JOptionPane.showMessageDialog(this, "Set parameters correctly", "DisChoco", JOptionPane.WARNING_MESSAGE);
			return;
		}
		SensorDCSPGenerator sn = new SensorDCSPGenerator(file, nbInstances, nbSensors, nbTargets);
		sn.setGeographyProperties(x_axis, y_axis);
		sn.setVisibilityProperties(minPv, maxPv, stepPv, minKv, maxKv, stepKv);
		sn.setCommunicationProperties(minPc, maxPc, stepPc, minKc, maxKc, stepKc);
		sn.setModel((String) type.getSelectedItem());
		sn.generate((String) format.getSelectedItem());
		file = null;
		JOptionPane.showMessageDialog(this, "Problems was generated with success");
		return;
	}

	/**
	 * This method initializes nbInstances
	 * 
	 * @return javax.swing.JSpinner
	 */
	private JSpinner getNbInstances() {
		if (nbInstances == null) {
			nbInstances = new JSpinner(new SpinnerNumberModel(100, 1, 1000, 1));
			nbInstances.setBounds(new Rectangle(360, 40, 60, 22));
		}
		return nbInstances;
	}

	/**
	 * This method initializes x_axis
	 * 
	 * @return javax.swing.JSpinner
	 */
	private JSpinner getX_axis() {
		if (x_axis == null) {
			x_axis = new JSpinner(new SpinnerNumberModel(5, 1, 100, 1));
			x_axis.setBounds(new Rectangle(200, 75, 60, 22));
		}
		return x_axis;
	}

	/**
	 * This method initializes y_axis
	 * 
	 * @return javax.swing.JSpinner
	 */
	private JSpinner getY_axis() {
		if (y_axis == null) {
			y_axis = new JSpinner(new SpinnerNumberModel(5, 1, 100, 1));
			y_axis.setBounds(new Rectangle(440, 75, 60, 22));
		}
		return y_axis;
	}

	/**
	 * This method initializes minKc
	 * 
	 * @return javax.swing.JSpinner
	 */
	private JSpinner getMinKc() {
		if (minKc == null) {
			minKc = new JSpinner(new SpinnerNumberModel(2, 1, 20, 1));
			minKc.setBounds(new Rectangle(220, 185, 60, 22));
		}
		return minKc;
	}

	/**
	 * This method initializes minKv
	 * 
	 * @return javax.swing.JSpinner
	 */
	private JSpinner getMinKv() {
		if (minKv == null) {
			minKv = new JSpinner(new SpinnerNumberModel(2, 1, 20, 1));
			minKv.setBounds(new Rectangle(220, 220, 60, 22));
		}
		return minKv;
	}

	/**
	 * This method initializes maxKc
	 * 
	 * @return javax.swing.JSpinner
	 */
	private JSpinner getMaxKc() {
		if (maxKc == null) {
			maxKc = new JSpinner(new SpinnerNumberModel(2, 2, 20, 1));
			maxKc.setBounds(new Rectangle(320, 185, 60, 22)); // Generated
		}
		return maxKc;
	}

	/**
	 * This method initializes stepKc
	 * 
	 * @return javax.swing.JSpinner
	 */
	private JSpinner getStepKc() {
		if (stepKc == null) {
			stepKc = new JSpinner(new SpinnerNumberModel(1, 1, 10, 1));
			stepKc.setBounds(new Rectangle(420, 185, 60, 22)); // Generated
		}
		return stepKc;
	}

	/**
	 * This method initializes maxKv
	 * 
	 * @return javax.swing.JSpinner
	 */
	private JSpinner getMaxKv() {
		if (maxKv == null) {
			maxKv = new JSpinner(new SpinnerNumberModel(2, 2, 20, 1));
			maxKv.setBounds(new Rectangle(320, 220, 60, 22)); // Generated
		}
		return maxKv;
	}

	/**
	 * This method initializes stepKv
	 * 
	 * @return javax.swing.JSpinner
	 */
	private JSpinner getStepKv() {
		if (stepKv == null) {
			stepKv = new JSpinner(new SpinnerNumberModel(1, 1, 10, 1));
			stepKv.setBounds(new Rectangle(420, 220, 60, 22)); // Generated
		}
		return stepKv;
	}

	private JComboBox getType() {
		if (type == null) {
			String[] formats = {GRID, REAL};
			type = new JComboBox(formats);
			type.setSelectedItem(GRID);//TODO
			type.setBounds(new Rectangle(100, 5, 140, 25));
		}
		return type;
	}

	private JComboBox getFormat() {
		if (format == null) {
			String[] formats = {Commands.SIMPLE, Commands.COMPLEX};
			format = new JComboBox(formats);
			format.setSelectedItem(Commands.COMPLEX);
			format.setBounds(new Rectangle(340, 5, 140, 25));
		}
		return format;
	}
}
