package kernel.order;

import java.util.Comparator;
import java.util.HashMap;

import util.IntArray;

/**
 * A Class for Min-Domain Degree ordering
 */
public class MaxDegPlusMinDomAO extends CentralizedAO implements ChainOrder {

	protected IntArray agentsId = null;
	protected Comparator<Integer> comparator = null;

	public MaxDegPlusMinDomAO() {
		super();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see order.CentralizedAgentOrdering#buildAgentsOrder()
	 */
	public IntArray buildAgentsOrder() {
		return this.buildAgentsOrderForComplex();
//		IntArray order = new IntArray();
//		int n = this.agentsComplexity.keySet().size();
//		double[] dom_deg = new double[n];
//		int i = 0;
//		Iterator<Integer> it = this.agentsComplexity.keySet().iterator();
//		while (it.hasNext()) {
//			int agId = it.next();
//			// agComp= domSize+","+nbConstraints+","+"nbVariables;
//			AgentComplexity agComp = this.agentsComplexity.get(agId);
//			if (agComp.getNumberConstraints() == 0)
//				dom_deg[i++] = Double.POSITIVE_INFINITY;
//			else
//				dom_deg[i++] = ((double) agComp.getMaxDomain()) / agComp.getNumberConstraints();
//		}
//		Object[] agIds = this.agentsComplexity.keySet().toArray();
//		for (i = 0 ; i < this.agentsComplexity.size() ; i++) {
//			int idx = util.Tools.indexOfMin(dom_deg);
//			dom_deg[idx] = Double.POSITIVE_INFINITY;
//			order.add((Integer) agIds[idx]);
//		}
//		return order;
	}
	
	/**
	 * 
	 */
	public IntArray buildAgentsOrderForComplex() {
		IntArray order = new IntArray();
		this.agentsId = new IntArray();
		for(int agId : this.agentsComplexity.keySet()){
			this.agentsId.add(agId);
//			int deg = agentsComplexity.get(agId).getNbNeighbors();
//			int dom = agentsComplexity.get(agId).getMaxDomain();
//			System.out.println("deg="+deg+" dom="+dom);
		}
		this.comparator = new Comparator<Integer>(){
			@Override
			public int compare(Integer agId1, Integer agId2) {
				int deg1 = agentsComplexity.get(agId1).getNbNeighbors();
				int deg2 = agentsComplexity.get(agId2).getNbNeighbors();
				int score = deg2 - deg1;
				if(score == 0){
					int dom1 = agentsComplexity.get(agId1).getMaxDomain();
					int dom2 = agentsComplexity.get(agId2).getMaxDomain();
					score = dom1 - dom2;
				}
				return score;
			}
			
		};
		this.agentsId.sort(this.comparator);
		while (this.agentsId.size()!=0) {
			int agId = this.agentsId.removeAt(0);
			order.add(agId);
			IntArray localAgents = this.agentsComplexity.get(agId).getLocalAgents();
			localAgents = localAgents.minus(order);
			if(!localAgents.isEmpty()){
				localAgents.sort(this.comparator);
				order.addAll(localAgents);
			}
		}
//		System.out.println(order);
//		System.exit(0);
		return order;
	}

	public static IntArray buildpriorityTree(final HashMap<Integer, AgentComplexity> complexities) {
		IntArray order = new IntArray();
		IntArray agentsId = new IntArray();
		for(int agId : complexities.keySet()){
			agentsId.add(agId);
		}
		Comparator<Integer> comparator = new Comparator<Integer>(){
			@Override
			public int compare(Integer agId1, Integer agId2) {
				int deg1 = complexities.get(agId1).getNbNeighbors();
				int deg2 = complexities.get(agId2).getNbNeighbors();
				int score = deg2 - deg1;
				if(score == 0){
					int dom1 = complexities.get(agId1).getMaxDomain();
					int dom2 = complexities.get(agId2).getMaxDomain();
					score = dom1 - dom2;
				}
				return score;
			}
			
		};
		agentsId.sort(comparator);
		while (agentsId.size()!=0) {
			int agId = agentsId.removeAt(0);
			order.add(agId);
			IntArray localAgents = complexities.get(agId).getLocalAgents();
			localAgents = localAgents.minus(order);
			if(!localAgents.isEmpty()){
				localAgents.sort(comparator);
				order.addAll(localAgents);
			}
		}
		return order;
	}

}
