package simple.dcsp.abtfamily.abtdo;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Random;

import util.IntArray;
/**
 * 
 * 
 * 
 * @authors 
 * <a href="mailto:ezzahir@ensa-agadir.ac.ma?subject=DisChoco &cc=wahbi@lirmm.fr &cc=bessiere@lirmm.fr">
 * Redoaune Ezzahir</a> and 
 * <a href="mailto:wahbi@lirmm.fr?subject=DisChoco &cc=ezzahir@ensa-agadir.ac.ma &cc=bessiere@lirmm.fr">
 * Mohamed Wahbi</a>
 *
 */
public class CurrentOrder implements Serializable {

	/**
	 * 
	 */
	private static final long serialVersionUID = 8522009754008718289L;
	private IntArray order;
	private HashMap<Integer, Integer> counters;

	public CurrentOrder(IntArray order) {
		this.order = order;
		this.counters = new HashMap<Integer, Integer>();
		for (int agId : this.order) {
			this.counters.put(agId, 0);
		}
	}

	public CurrentOrder(IntArray order, HashMap<Integer, Integer> counters) {
		this.order = order;
		this.counters = counters;
	}

	@SuppressWarnings("unchecked")
	public CurrentOrder clone() {
		HashMap<Integer, Integer> counters = (HashMap<Integer, Integer>) this.counters.clone();
		return new CurrentOrder(this.order.clone(), counters);
	}

	public IntArray getOrder() {
		return this.order;
	}

	public String toString() {
		String order = "[";
		for (int agId : this.order) {
			int counter = this.counters.get(agId);
			order += "(" + agId + "," + counter + ");";
		}
		order = order.substring(0, order.length() - 1);
		order += "]";
		return order;
	}

	// public static CurrentOrder fromString(String str) {
	// IntArray order = new IntArray();
	// HashMap<Integer, Integer> counters = new HashMap<Integer, Integer>();
	// str = str.substring(1, str.indexOf("]"));
	// StringTokenizer tok = new StringTokenizer(str, ";");
	// while (tok.hasMoreTokens()) {
	// String elements = tok.nextToken();
	// elements = elements.substring(1, elements.length() - 1);
	// String[] element = elements.split(",");
	// int agId = Integer.parseInt(element[0]);
	// int counter = Integer.parseInt(element[1]);
	// order.add(agId, order.size());
	// counters.put(agId, counter);
	// }
	// return new CurrentOrder(order, counters);
	// }

	public void setNewOrder(int myId, IntArray newOrder) {
		this.order = newOrder;
		this.updateCounters(myId);
	}

	public void updateCounters(int myId) {
		IntArray lowerAgents = this.order.getElementsAfter(myId);
		for (int lower : lowerAgents) {
			this.counters.put(lower, 0);
		}
		this.counters.put(myId, this.counters.get(myId) + 1);
	}

	/**
	 * 
	 * @param currentOrder
	 * @return <b>(i+1)</b> if <code>this</code> order is the newest</br> <b>0</b> if this order and <code>currentOrder</code> are equal</br>
	 *         <b>-(i+1)</b> if <code>currentOrder</code> is the newest</br>
	 */
	public int compare(CurrentOrder currentOrder) {
		for (int i = 0 ; i < this.order.size() ; i++) {
			int agId1 = this.order.get(i);
			int counter1 = this.counters.get(agId1);
			int agId2 = currentOrder.order.get(i);
			int counter2 = currentOrder.counters.get(agId2);
			if (counter1 > counter2) { return (i + 1); }
			if (counter1 < counter2) { return -(i + 1); }
		}
		return 0;
	}

	public CurrentOrder getNgTriggeredOrder(int myId, int agId) {
		CurrentOrder newOrder = this.clone();
		int myRank = newOrder.order.indexOf(myId);
		int agRank = newOrder.order.indexOf(agId);

		for (int i = agRank ; i > myRank + 1 ; i--) {
			newOrder.order.add(newOrder.order.get(i - 1), i);
		}
		newOrder.order.add(agId, myRank + 1);
		newOrder.updateCounters(myId);
		return newOrder;
	}

	public CurrentOrder getRandomOrder(int myId) {
		CurrentOrder newOrder = this.clone();
		IntArray lowerAgents = newOrder.order.getElementsAfter(myId);
		Random rand = new Random(System.currentTimeMillis());
		for (int i = newOrder.order.indexOf(myId) + 1 ; i < newOrder.order.size() ; i++) {
			int index = rand.nextInt(lowerAgents.size());
			int agId = lowerAgents.get(index);
			lowerAgents.remove(agId);
			newOrder.order.add(agId, i);
		}
		newOrder.updateCounters(myId);
		return newOrder;
	}

	public CurrentOrder getDomainSizeOrder(int myId, HashMap<Integer, Integer> domainSizes) {
		CurrentOrder newOrder = this.clone();
		IntArray lowerAgents = newOrder.order.getElementsAfter(myId);
		int[] agents = sort(lowerAgents.clone(), domainSizes);
		for (int i = 0 ; i < lowerAgents.size() ; i++) {
			int agId = agents[i];
			newOrder.order.add(agId, i + newOrder.order.indexOf(myId) + 1);
		}
		newOrder.updateCounters(myId);
		return newOrder;
	}

	public CurrentOrder getSingleRandomOrder(int myId) {
		CurrentOrder newOrder = this.clone();
		Random rand = new Random(System.currentTimeMillis());
		IntArray lowerAgents = newOrder.order.getElementsAfter(myId);
		int index = rand.nextInt(lowerAgents.size());
		int agId = lowerAgents.get(index);
		int myRank = newOrder.order.indexOf(myId);
		int agRank = newOrder.order.indexOf(agId);
		for (int i = agRank ; i > myRank + 1 ; i--) {
			newOrder.order.add(newOrder.order.get(i - 1), i);
		}
		newOrder.order.add(agId, myRank + 1);
		newOrder.updateCounters(myId);
		return newOrder;
	}

	public CurrentOrder getSingleDomainSizeOrder(int myId, HashMap<Integer, Integer> domainSizes) {
		CurrentOrder newOrder = this.clone();
		IntArray lowerAgents = newOrder.order.getElementsAfter(myId);
		if (lowerAgents.size() == 0) {
			newOrder.updateCounters(myId);
			return newOrder;
		}
		int[] agents = sort(lowerAgents.clone(), domainSizes);
		int agId = agents[0];
		int myRank = newOrder.order.indexOf(myId);
		int agRank = newOrder.order.indexOf(agId);
		for (int i = agRank ; i > myRank + 1 ; i--) {
			newOrder.order.add(newOrder.order.get(i - 1), i);
		}
		newOrder.order.add(agId, myRank + 1);
		newOrder.updateCounters(myId);
		return newOrder;
	}

	public static int[] sort(IntArray agents, HashMap<Integer, Integer> domainSizes) {
		int[] ans = new int[agents.size()];
		for (int i = 0 ; i < ans.length ; i++) {
			int minId = agents.get(0);
			int minSize = domainSizes.get(minId);
			for (int agId : agents) {
				if (minSize > domainSizes.get(agId)) {
					minSize = domainSizes.get(agId);
					minId = agId;
				}
			}
			ans[i] = minId;
			agents.remove(minId);
		}
		return ans;
	}

	public boolean equals(Object obj) {
		CurrentOrder arg = (CurrentOrder) obj;
		return (arg.order.compare(this.order) == 0 && this.compare(arg) == 0);
	}

	public static void main(String[] agrs) {
		IntArray order = new IntArray();
		order.add(1);
		order.add(2);
		order.add(3);
		order.add(4);
		order.add(5);
		HashMap<Integer, Integer> domainSizes = new HashMap<Integer, Integer>();
		domainSizes.put(1, 5);
		domainSizes.put(2, 2);
		domainSizes.put(3, 3);
		domainSizes.put(4, 4);
		domainSizes.put(5, 1);
		System.out.println(order + "\n" + domainSizes);
		int[] agents = order.getArray();
		for (int i = 0 ; i < agents.length ; i++) {
			System.out.print(agents[i] + " ");
		}
		System.out.println();
		agents = sort(order, domainSizes);
		for (int i = 0 ; i < agents.length ; i++) {
			System.out.print(agents[i] + " ");
		}
		System.out.println();
	}

	public HashMap<Integer, Integer> getCounters() {
		return this.counters;
	}

}
