package simple.dcsp.abtfamily.abtdoretroz;

import java.util.HashMap;
import java.util.Iterator;

import kernel.MasterObserver;
import kernel.com.Message;
import util.Commands;
import util.Instantiation;
import util.IntArray;
import util.Logger;

/**
 * termination detection (Silaghi et al. 2000)
 * 
 * @author redouane
 * 
 */
public class TerminationDetector extends MasterObserver {

	protected HashMap<Integer, Instantiation> acceptedMsg;
	protected HashMap<Integer, CurrentOrder> currentOrders = null;

	protected IntArray initialEndAgents = new IntArray();
	protected CurrentOrder bestOrder = null;

	public TerminationDetector() {
		super();
		this.acceptedMsg = new HashMap<Integer, Instantiation>();
		this.currentOrders = new HashMap<Integer, CurrentOrder>();
	}

	@Override
	public boolean handleReceivedMessage(Message msg) {
		if (msg.getType().equals(Commands.ORDER)) {
			this.ProcessOrder(msg);
			this.checkEnd();
		}
		if (msg.getType().equals(Commands.TERMINATION_DETETION)) {
			Instantiation accepted = (Instantiation) msg.get(Commands.ACCEPTED);
			CurrentOrder rcvOrder = (CurrentOrder) msg.get(Commands.ORDER); 
			int senderId = msg.getSenderId();
			this.acceptedMsg.remove(senderId);
			this.currentOrders.put(senderId, rcvOrder);
			this.acceptedMsg.put(senderId, accepted);
			this.setBestOrder(rcvOrder);
			this.deleteObsoleteInformations();
			this.checkEnd();
		}
		return true;
	}

	private void deleteObsoleteInformations() {
		Iterator<Integer> it = this.currentOrders.keySet().iterator();
		while (it.hasNext()) {
			int agId = it.next();
			CurrentOrder order = this.currentOrders.get(agId);
			int cmp = this.bestOrder.compare(order);
			int index = this.bestOrder.getOrder().indexOf(agId);
			if (cmp != 0 && Math.abs(cmp) <= index + 1) {
				it.remove();
				this.acceptedMsg.remove(agId);
			}
		}
	}

	private void ProcessOrder(Message msg) {
		CurrentOrder rcvOrder = (CurrentOrder) msg.get(Commands.ORDER); // CurrentOrder.fromString(""+msg.get(Commands.ORDER));
		this.setBestOrder(rcvOrder);
	}

	private IntArray computeInitialEndAgents() {
		this.initialEndAgents = new IntArray();
		for (int agId : this.bestOrder.getOrder()) {
			if (this.GammaPlus(agId).size() == 0) {
				this.initialEndAgents.add(agId);
			}
		}
		this.master.log("Roots agents : "+this.initialEndAgents+" ", Logger.INFO_LEVEL);
		return this.initialEndAgents;
	}

	/**
	 * 
	 * @param agId
	 * @return
	 */
	private IntArray GammaPlus(int agId) {
		IntArray higherAgents = this.bestOrder.getOrder().getElementsBefor(agId);
		IntArray neighbours = this.master.getNeighbors(agId);
		return IntArray.intersection(higherAgents, neighbours);
	}

	/**
	 * 
	 * @return
	 */
	private boolean checkEnd() {
		this.computeInitialEndAgents();
		if (this.initialEndAgents.size() > this.currentOrders.size() || this.initialEndAgents.size() > this.acceptedMsg.size()) { return false; }
		this.master.log("I check the end\n"+
				"Roots agents : "+this.initialEndAgents+"\n"+
				"bestOrder : "+this.bestOrder+
				"acceptedMsg : "+this.acceptedMsg+
				"orders    : "+this.currentOrders
				, Logger.INFO_LEVEL);

		Instantiation solution = new Instantiation();
		for (int agId : this.initialEndAgents) {
			if (!this.currentOrders.containsKey(agId) || !this.acceptedMsg.containsKey(agId)) {
				this.master.log("orders or acceptedMsg do not contains "+agId, Logger.INFO_LEVEL);
				return false;
			}
			CurrentOrder order = this.currentOrders.get(agId);
			int cmp = this.bestOrder.compare(order);
			if (cmp != 0 && Math.abs(cmp) <= this.bestOrder.getOrder().indexOf(agId) + 1) {
				this.master.log("problem in orders \n"+agId+" => "+order +" and \n"+this.bestOrder, Logger.INFO_LEVEL);
				return false;
			}
			Instantiation accepted = this.acceptedMsg.get(agId);
			if (solution.compatible(accepted)) {
				solution.priorityMerge(accepted);
			} else {
				this.master.log("non compatible accepted "+solution+" \n"+accepted, Logger.INFO_LEVEL);
				return false;
			}
		}
		this.master.log("terminate with SUCCESS : "+solution, Logger.INFO_LEVEL);
		this.master.setEndState(Commands.SUCCESS);
		this.master.broadcastSystemMessage(0);
		return true;
	}

	@Override
	public void handleSendMessage(Message msg) {}

	public void setBestOrder(CurrentOrder currentOrder) {
		if (this.bestOrder == null) {
			this.bestOrder = currentOrder;
			this.master.log("The best order "+currentOrder+" ", Logger.INFO_LEVEL);
		} else {
			if (this.bestOrder.compare(currentOrder) < 0) {
				this.bestOrder = currentOrder;
				this.master.log("The best order "+currentOrder+" ", Logger.INFO_LEVEL);
			}
		}
	}

}
