package simple.dcsp.afcfamily.afcng.approximateDom;

import java.util.HashMap;
import java.util.Iterator;

import kernel.Constraint;
import kernel.StopException;
import kernel.com.Message;
import kernel.constraints.integer.ExtensionBinConstraint;
import kernel.order.AgentOrdering;
import kernel.vars.integer.IntDomainIterator;
import simple.dcsp.afcfamily.afcng.DynamicAFCngAgent;
import simple.dcsp.afcfamily.afcng.TimeStamp;
import util.Instantiation;
import util.IntArray;
import util.Tools;

/**
 * 
 * 
 * @authors 
 * <a href="mailto:ezzahir@ensa-agadir.ac.ma?subject=DisChoco &cc=wahbi@lirmm.fr &cc=bessiere@lirmm.fr">
 * Redoaune Ezzahir</a> and 
 * <a href="mailto:wahbi@lirmm.fr?subject=DisChoco &cc=ezzahir@ensa-agadir.ac.ma &cc=bessiere@lirmm.fr">
 * Mohamed Wahbi</a>
 *
 */
public class AFCngApproximateMinDomAgent extends DynamicAFCngAgent {

	private static final String LowerBound = "LB";
	private static final String UpperBound = "UB"; 
	private short type;

	/**
	 * the number of conflicts for each value on each neighbor domain. 
	 */
	private HashMap<Integer, HashMap<Integer, Integer>> nbConflicts = null;
	
	private HashMap<Integer, Integer> lbDomais;
	private HashMap<Integer, Integer> ubDomais;
	private HashMap<Integer, Integer> LB;
	private HashMap<Integer, Integer> UB;
	private HashMap<Integer, Integer> initialDomains = null;

	public AFCngApproximateMinDomAgent(int agentId, String agName, String description, short type) {
		super(agentId, agName, description);
		this.type = type;
		this.lbDomais = new HashMap<Integer, Integer>();
		this.ubDomais = new HashMap<Integer, Integer>();
		this.initialDomains = new HashMap<Integer, Integer>();
		this.cpaMsgH = new EstCPAMsgHandler(this);
		this.nbConflicts = new HashMap<Integer, HashMap<Integer, Integer>>();
		this.agentOrder = new AgentOrdering(this) {
			public void processOrder(Message msg) {
				super.processOrder(msg);
				for (int agId : this.agentsOrder) {
					//init by domain sizes
					int dom = (Integer) msg.get(agId + "");
					lbDomais.put(agId, 0);
					ubDomais.put(agId, 0);
					initialDomains.put(agId, dom);
				}
			}
		};
	}

	@Override
	public void configurePropagationEngine() { 
//		this.eventsConfig.get(VarEventMask.LOCAL_VAR_IS_FIRST).add(VariableEvent.INSTANCIATE);
//		this.eventsConfig.get(VarEventMask.LOCAL_VAR_IS_FIRST).add(VariableEvent.DESINSTANCIATE);
		super.configurePropagationEngine();
		IntDomainIterator itDom = this.var.getDomain().iterator();
		while(itDom.hasNext()){
			int value = itDom.next();
			this.nbConflicts.put(value, new HashMap<Integer, Integer>());
		}
		Iterator<Constraint> it = this.constraintHandler.getConstraintsIterator();
		while(it.hasNext()){
			Constraint c = it.next();
			if(c instanceof ExtensionBinConstraint){
				ExtensionBinConstraint constraint = ((ExtensionBinConstraint) c);
				itDom = this.var.getDomain().iterator();
				while(itDom.hasNext()){
					int value = itDom.next();
					int agId = constraint.getExternalVar(0).getAgentId();
					int nbConflict = constraint.getNumberOfConflicts(value);
					this.nbConflicts.get(value).put(agId, nbConflict);
				}
			} else {
				//TODO get number of conflicts
//				System.out.println("Unkwon number of Conflicts.\n"+this.agentId+" : "+c);
//				System.exit(0);
			}
		}
	}
	
	/**
	 * 
	 */
	@Override
	public void sendCPA(Instantiation cpa) throws StopException {
		this.computeBounds(cpa);
		super.sendCPA(cpa);
	}

	/**
	 * min_i is set to the maximum number of inconsistent values of variable x_i 
	 * with the value of every assigned variable.
	 * 
	 * max_i is set to the sum of all inconsistent values of variable x_i with 
	 * respect to each assigned variable
	 * 
	 * Then, the next variable to be assigned is chosen as follows: 
	 * (i) if there is xi such that min_i >= min{d, max_j}, forall x_j unassigned, 
	 * select x_i (where d is the domain size); 
	 * (ii) otherwise, select the variable with maximum max_j .
	 * 
	 */
	private void computeBounds(Instantiation cpa) {
		this.LB = new HashMap<Integer, Integer>();
		this.UB = new HashMap<Integer, Integer>();
		IntArray unassigned = this.agentOrder.getOrder();
		unassigned = unassigned.minus(cpa.getAgentIds());
		int myValue = this.var.getValue();
		for (int lower : unassigned) {
			if(this.communicator.isNeighbor(lower)){
				// compute the new lower bound 
				// lb_j <-- max(lb_j, conflicts<x_i,x_j>)
				int nbConflict = this.nbConflicts.get(myValue).get(lower);
				int lb = this.lbDomais.get(lower);
				LB.put(lower, Math.max(lb, nbConflict));
				// ub_j <-- min(|Dj|,ub_j + conflicts<x_i,x_j>)
				int ub = this.ubDomais.get(lower) + nbConflict;
				UB.put(lower, Math.min(initialDomains.get(lower), ub));
			} else {
				LB.put(lower, this.lbDomais.get(lower));
				UB.put(lower, this.ubDomais.get(lower));
			}
		}
	}

	/**
	 * get the next agent to which xi must send the (to assign) 
	 */
	public int getNextAgent() {//TODO
		String s = "nbConflicts: "+this.nbConflicts+"\n"+
		"LB: "+this.LB+"\n"+
		"UB: "+this.UB+"\n"+
		"lbDomais: "+this.lbDomais+"\n"+
		"ubDomais: "+this.ubDomais+"\n";
		this.log(s, util.Logger.INFO_LEVEL);
		switch (this.type) {
			case AFCngApproximateMinDomMaster.MinDom : 
				return this.getMinDomainNextAgent();
			case AFCngApproximateMinDomMaster.MinDomDeg :
				return this.getDomDegNextAgent();
			default : return -1;
		}
	}

	/**
	 * Then, the next variable to be assigned is chosen as follows: 
	 * (i) if there is xi such that min_i >= min{d, max_j}, forall x_j unassigned, 
	 * select x_i (where d is the domain size); 
	 * (ii) otherwise, select the variable with maximum max_j .
	 * 
	 * @return next agent to assign
	 */
	public int getMinDomainNextAgent() {//TODO
		if (UB.isEmpty())
			return -1;
		int idMinUB = Tools.MinOf(UB);
		int idmaxLB = Tools.MaxOf(LB);
		if(LB.get(idmaxLB) > UB.get(idMinUB)){
			return idmaxLB;
		} else {
			return Tools.MinOf(LB);
		}
	}

	/**
	 * @param bounds
	 * @return next agent to assign
	 */
	public int getDomDegNextAgent() {
		IntArray assigned = this.agentView.toInstantiation().getAgentIds();
		assigned.add(this.agentId);
		IntArray unassigned = this.agentOrder.getOrder().minus(assigned);
		if (unassigned.size() == 0 || UB.isEmpty())
			return -1;
		HashMap<Integer, Integer> degrees = this.getDegrees(unassigned);
		for (int agId : UB.keySet()) {
			int deg = degrees.get(agId);
			double upperBound = UB.get(agId);
			if (upperBound == 0) {
				return agId;
			} else {
				if (deg == 0) {
					upperBound = Double.POSITIVE_INFINITY;
				} else {
					upperBound /= deg;
				}
				boolean selected = true;
				for (int id : LB.keySet()) {
					int degree = degrees.get(id);
					double lowerBound = LB.get(id);
					if (degree == 0) {
						lowerBound = Double.POSITIVE_INFINITY;
					} else {
						lowerBound /= degree;
					}
					if (agId != id) {
						if (upperBound > lowerBound) {
							selected = false;
							break;
						}
					}
				}
				if (selected) { return agId; }
			}
		}
		int IdOfMinLb = -1;
		double minLb = Double.POSITIVE_INFINITY;
		for (int agId : LB.keySet()) {
			int deg = degrees.get(agId);
			double lowerBound = LB.get(agId);
			if (deg == 0)
				lowerBound = Double.POSITIVE_INFINITY;
			else
				lowerBound /= deg;
			if (lowerBound < minLb) {
				IdOfMinLb = agId;
				minLb = LB.get(agId);
			}
		}
		return IdOfMinLb;
	}

	private HashMap<Integer, Integer> getDegrees(IntArray unassigned) {
		HashMap<Integer, Integer> map = new HashMap<Integer, Integer>();
		for (int agId : UB.keySet()) {
			map.put(agId, ddegree(agId, unassigned));
		}
		return map;
	}

	private int ddegree(int agId, IntArray unassigned) {
		int deg = 0;
		Iterable<Integer> neighbors = this.agentOrder.getAgentsGraph().getNeighbors(agId);
		for (int id : neighbors)
			if (unassigned.contains(id)) {
				deg++;
			}
		return deg;
	}

	// ====================================================================================
	//
	// ====================================================================================

	/**
	 *  Approximated MinDom CPA message handler
	 */
	public class EstCPAMsgHandler extends CPAMsgHandler {

		public EstCPAMsgHandler(AFCngApproximateMinDomAgent algo) {
			super(algo);
		}

		@SuppressWarnings("unchecked")
		public void processMessage(Message msg) throws StopException {
			int nextId = (Integer) msg.get(NEXT_ID);
			TimeStamp ts = (TimeStamp) msg.get(TIMESTAMP);
			if (timeStamp.compareTimeStamp(ts) > 0) {
				if (this.myAgent.getAgentId() == nextId) {
					HashMap<Integer, Integer> lb = (HashMap<Integer, Integer>) msg.get(LowerBound);
					HashMap<Integer, Integer> ub = (HashMap<Integer, Integer>) msg.get(UpperBound);
					lbDomais.putAll(lb);
					ubDomais.putAll(ub);
				}
				Instantiation cpa = (Instantiation) msg.get(CPA);
				receiveCPA(ts, cpa, nextId);
			}
		}
		
		/**
		 * 
		 */
		@Override
		public void sendCPAMessage(Instantiation cpa, int receiverId, int nextAgentId) {
			if (receiverId != nextAgentId) {
				super.sendCPAMessage(cpa, receiverId, nextAgentId);
			} else {
				Message msg = createMessage();
				msg.addContents(CPA, cpa.clone());
				msg.addContents(NEXT_ID, nextAgentId);
				msg.addContents(TIMESTAMP, timeStamp.clone());
				msg.addContents(LowerBound, LB);
				msg.addContents(UpperBound, UB);
				msg.setReceiverId(receiverId);
				sendMessage(msg);
			}
		}
	}
}
