package simple.dcsp.agile.exp;

import kernel.StopException;
import kernel.com.Message;
import kernel.com.MessageHandler;
import simple.Nogood;
import util.Commands;
import util.Instantiation;
import util.IntArray;
import util.VarAssignment;

/**
 * for each value we store just one Nogood following this heuristic: when comparing two nogoods, select the one with the highest possible lowest
 * variable involved (Nogood Selector HPLV)
 */

public class NogoodMsgHandler extends MessageHandler {

	public NogoodMsgHandler(AgileAgent agent) {
		super(agent, Commands.NOGOOD);
	}

	/**
	 * 
	 */
	public void processMessage(Message msg) throws StopException {
		int senderId = msg.getSenderId();
		IntArray order = (IntArray) msg.get(Commands.ORDER);
		double[] TV = (double[]) msg.get(AgileAgent.TV);
		int weight = (Integer) msg.get(Commands.WEIGHT);
		((AgileAgent) this.myAgent).setWeights(msg.getSenderId(), weight);
		((AgileAgent) this.myAgent).checkOrder(order, TV);
		Nogood ng = (Nogood) msg.get(Commands.NOGOOD);
		((AgileAgent) this.myAgent).updateAgentView(ng.getLhs());
		if (this.isCoherent(ng.getLhs(), ng.getRhs()) && this.isCompatibleWithMyOrder(ng)) {
			// ExplainedIntVar v = (ExplainedIntVar) ((AgileABTAgent) this.myAgent).var;
			((AgileAgent) this.myAgent).makeNoGoodConstraint(ng);
			((AgileAgent) this.myAgent).CheckAddLink(ng.getLhs());
		} else {
			this.myAgent.auxiliaryStatistics.get(Commands.nbObsoleteNogood).value++;
			if (!((AgileAgent) this.myAgent).IchangeMyOrder) {
				boolean value = ng.getRhsValue() == ((AgileAgent) this.myAgent).var.getValue();
				boolean isLower = this.myAgent.agentOrder.isLower(senderId);
				if (value && isLower) {
					Instantiation lhs = ((AgileAgent) this.myAgent).getExplanationLhs();
					int rhs = ((AgileAgent) this.myAgent).getExplanationRhs();
					((AgileAgent) this.myAgent).okMsgH.sendOkMessage(senderId, lhs, rhs);
				}
			}
		}
		((AgileAgent) this.myAgent).CheckAgentView();
	}

	/**
	 * @param nogood
	 */
	public void sendNogood(Nogood nogood) {
		Message msg = this.createMessage();
		IntArray order = this.myAgent.getOrder();
		double[] tv = ((AgileAgent) this.myAgent).myTV.clone();
		msg.addContents(Commands.NOGOOD, nogood);
		msg.addContents(Commands.ORDER, order);
		msg.addContents(AgileAgent.TV, tv);
		msg.addContents(Commands.WEIGHT, ((AgileAgent) this.myAgent).getMyWeight());
		msg.setReceiverId(nogood.getRhsAgentId());
		((AgileAgent)this.myAgent).storeStatisticsFor(msg);
		this.myAgent.sendMessage(msg);
	}

	/**
	 * @param ng
	 * @param rhs
	 * @return
	 */
	private boolean isCompatibleWithMyOrder(Nogood ng) {
		IntArray higherAgs = this.myAgent.agentOrder.getHigherAgents();
		return higherAgs.contains(ng.getLhs().getAgentIds());
	}

	/**
	 * @param rhs
	 * @param nogood
	 * @return
	 */
	public boolean isCoherent(Instantiation lhs, VarAssignment rhs) {
		int myValue = ((AgileAgent) this.myAgent).var.getValue();
		return this.myAgent.agentView.compatibleWith(lhs) && rhs.getValue() == myValue;
	}

}
