package simple.dcsp.agileabt.explanation;

import kernel.StopException;
import kernel.com.Message;
import kernel.com.MessageHandler;
import simple.Nogood;
import util.Commands;
import util.Instantiation;
import util.IntArray;
import util.VarAssignment;

/**
 * for each value we store just one Nogood following this heuristic: when comparing two nogoods, select the one with the highest possible lowest
 * variable involved (Nogood Selector HPLV)
 */

public class NogoodMsgHandler extends MessageHandler {

	public NogoodMsgHandler(AgileABTAgent agent) {
		super(agent, Commands.NOGOOD);
	}

	/**
	 * 
	 */
	public void processMessage(Message msg) throws StopException {
		int senderId = msg.getSenderId();
		IntArray order = (IntArray) msg.get(Commands.ORDER);
		double[] TV = (double[]) msg.get(AgileABTAgent.TV);
		((AgileABTAgent) this.myAgent).checkOrder(order, TV);
		Nogood ng = (Nogood) msg.get(Commands.NOGOOD);
		((AgileABTAgent) this.myAgent).updateAgentView(ng.getLhs());
		if (this.isCoherent(ng.getLhs(), ng.getRhs()) && this.isCompatibleWithMyOrder(ng)) {
			// ExplainedIntVar v = (ExplainedIntVar) ((AgileABTAgent) this.myAgent).var;
			((AgileABTAgent) this.myAgent).makeNoGoodConstraint(ng);
			((AgileABTAgent) this.myAgent).CheckAddLink(ng.getLhs());
		} else {
			this.myAgent.auxiliaryStatistics.get(Commands.nbObsoleteNogood).value++;
			if (!((AgileABTAgent) this.myAgent).IchangeMyOrder) {
				boolean value = ng.getRhsValue() == ((AgileABTAgent) this.myAgent).var.getValue();
				boolean isLower = this.myAgent.agentOrder.isLower(senderId);
				if (value && isLower) {
					Explanation myExp = ((AgileABTAgent) this.myAgent).getMyExplanation();
					((AgileABTAgent) this.myAgent).okMsgH.sendOkMessage(senderId, myExp);
				}
			}
		}
		((AgileABTAgent) this.myAgent).CheckAgentView();
	}

	/**
	 * @param nogood
	 */
	public void sendNogood(Nogood nogood) {
		// String myTV = Utility.arrayToString(((AgileABTAgent)this.myAgent).myTV);
		Message msg = this.createMessage();
		msg.addContents(Commands.NOGOOD, nogood);
		msg.addContents(Commands.ORDER, this.myAgent.getOrder());
		msg.addContents(AgileABTAgent.TV, ((AgileABTAgent) this.myAgent).myTV.clone());
		msg.setReceiverId(nogood.getRhsAgentId());
		((AgileABTAgent) this.myAgent).storeStatisticsFor(msg);
		this.myAgent.sendMessage(msg);
	}

	/**
	 * @param ng
	 * @param rhs
	 * @return
	 */
	private boolean isCompatibleWithMyOrder(Nogood ng) {
		IntArray higherAgs = this.myAgent.agentOrder.getHigherAgents();
		return higherAgs.contains(ng.getLhs().getAgentIds());
	}

	/**
	 * @param rhs
	 * @param nogood
	 * @return
	 */
	public boolean isCoherent(Instantiation lhs, VarAssignment rhs) {
		int myValue = ((AgileABTAgent) this.myAgent).var.getValue();
		return this.myAgent.agentView.compatibleWith(lhs) && rhs.getValue() == myValue;
	}

}
